"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeoLocation = void 0;
const fs_1 = require("fs");
const net_1 = require("net");
const utils_1 = require("../utils");
const CountryCode_1 = require("./CountryCode");
class GeoLocation {
    /**
       * Create GeoLocation example
       * @example
       * new GeoLocation("Geo.dat")
       * @param datPath
       */
    constructor(datPath) {
        this.datPath = datPath;
        this.country = new CountryCode_1.default();
        this.datData = Buffer.alloc(0);
        this.initStatus = false;
    }
    _readLine(type, ip) {
        let offset = 0;
        do {
            let dataBuffer = this.datData.subarray(offset, offset + 3);
            if (dataBuffer[0] === 10) {
                if (dataBuffer[1] === 4) {
                    dataBuffer = this.datData.subarray(offset, offset + 9);
                    const countryCode = dataBuffer.subarray(7, 9).readInt16BE();
                    if (type === 4 && (0, utils_1.isIPInCIDR)(ip, (0, utils_1.buffer2CRID)(dataBuffer.subarray(2, 7)))) {
                        return countryCode;
                    }
                    offset += 9;
                }
                else if (dataBuffer[1] === 6) {
                    const len = dataBuffer[2] * 2;
                    // readSync(this.openFileId, dataBuffer, { position: offset, length: len + 6 });
                    dataBuffer = this.datData.subarray(offset, offset + len + 6);
                    // console.log(len, dataBuffer);
                    const maskBuffer = dataBuffer.subarray(3 + len, 3 + len + 1).readUint8();
                    const ipBuf = dataBuffer.subarray(3, 3 + len);
                    const countryCode = dataBuffer.subarray(4 + len, 6 + len).readInt16BE();
                    const ipAddr = [];
                    for (let i = 0; i < ipBuf.length; i += 2) {
                        // console.log(i, ipBuf.subarray(i, i + 2));
                        ipAddr.push(ipBuf.subarray(i, i + 2).readUInt16BE().toString(16));
                    }
                    const cidr = ipAddr.join(':') + '::/' + maskBuffer;
                    // console.log(countryCode, cidr);
                    if (type === 6 && (0, utils_1.isIPv6InCIDR)(ip, cidr)) {
                        return countryCode;
                    }
                    offset += len + 6;
                }
                else {
                    throw new Error(`[GeoLocation] dat file error type = ${dataBuffer[1]} offset = ${offset + 1}`);
                }
            }
            else {
                throw new Error(`[GeoLocation] dat file error header = ${dataBuffer[0]} offset = ${offset}`);
            }
        } while (offset < this.datData.length);
        return 0;
    }
    /**
       * Load dat file to memory
       * @param datFile
       * @returns {number}
       */
    load(datFile) {
        if (datFile)
            this.datPath = datFile;
        if ((0, fs_1.existsSync)(this.datPath)) {
            this.datData = (0, fs_1.readFileSync)(this.datPath);
        }
        else {
            throw new Error(`[GeoLocation] does not exist: ${this.datPath}`);
        }
        this.initStatus = true;
        return this.datData.length;
    }
    // /**
    //  * Country id get name
    //  * @example
    //  * GeoLocation.code2Location(1)
    //  * @param code
    //  * @returns {string}
    //  */
    // public code2Country(code: number): string {
    //   return Object.keys(country).filter(key => country[key] === code).shift() || 'unknown';
    // }
    getIpv4CIDR(address) {
        if (this.initStatus && this.isIpv4(address)) {
            return this._readLine(4, address);
        }
        else {
            return -1;
        }
    }
    getIpv6CIDR(address) {
        if (this.initStatus && this.isIpv6(address)) {
            return this._readLine(6, address);
        }
        else {
            return -1;
        }
    }
    /**
       * Check IP is ipv6
       * @example
       * GeoLocation.isIpv4('127.0.0.1')
       * @param addr
       */
    isIpv4(addr) {
        return (0, net_1.isIPv4)(addr);
    }
    /**
       * Check IP is ipv6
       * @example
       * GeoLocation.isIpv6('2409:8c54:870:310:0:ff:b0ed:40ac')
       * @param addr
       */
    isIpv6(addr) {
        return (0, net_1.isIPv6)(addr);
    }
}
exports.GeoLocation = GeoLocation;
